package com.ruoyi.system.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.ZsArticleVo;
import com.ruoyi.system.domain.bo.ZsArticleBo;
import com.ruoyi.system.service.IZsArticleService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 站点文章管理
 *
 * @author 王强
 * @date 2025-04-29
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/article")
public class ZsArticleController extends BaseController {

    private final IZsArticleService iZsArticleService;

    /**
     * 查询站点文章管理列表
     */
    @SaCheckPermission("system:article:list")
    @GetMapping("/list")
    public TableDataInfo<ZsArticleVo> list(ZsArticleBo bo, PageQuery pageQuery) {
        return iZsArticleService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出站点文章管理列表
     */
    @SaCheckPermission("system:article:export")
    @Log(title = "站点文章管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(ZsArticleBo bo, HttpServletResponse response) {
        List<ZsArticleVo> list = iZsArticleService.queryList(bo);
        ExcelUtil.exportExcel(list, "站点文章管理", ZsArticleVo.class, response);
    }

    /**
     * 获取站点文章管理详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:article:query")
    @GetMapping("/{id}")
    public R<ZsArticleVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iZsArticleService.queryById(id));
    }

    /**
     * 新增站点文章管理
     */
    @SaCheckPermission("system:article:add")
    @Log(title = "站点文章管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody ZsArticleBo bo) {
        return toAjax(iZsArticleService.insertByBo(bo));
    }

    /**
     * 修改站点文章管理
     */
    @SaCheckPermission("system:article:edit")
    @Log(title = "站点文章管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody ZsArticleBo bo) {
        return toAjax(iZsArticleService.updateByBo(bo));
    }

    /**
     * 删除站点文章管理
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:article:remove")
    @Log(title = "站点文章管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iZsArticleService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
