package com.ruoyi.system.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.ZsBannerVo;
import com.ruoyi.system.domain.bo.ZsBannerBo;
import com.ruoyi.system.service.IZsBannerService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 站点banner管理
 *
 * @author ruoyi
 * @date 2025-04-28
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/banner")
public class ZsBannerController extends BaseController {

    private final IZsBannerService iZsBannerService;

    /**
     * 查询站点banner管理列表
     */
    @SaCheckPermission("system:banner:list")
    @GetMapping("/list")
    public TableDataInfo<ZsBannerVo> list(ZsBannerBo bo, PageQuery pageQuery) {
        return iZsBannerService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出站点banner管理列表
     */
    @SaCheckPermission("system:banner:export")
    @Log(title = "站点banner管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(ZsBannerBo bo, HttpServletResponse response) {
        List<ZsBannerVo> list = iZsBannerService.queryList(bo);
        ExcelUtil.exportExcel(list, "站点banner管理", ZsBannerVo.class, response);
    }

    /**
     * 获取站点banner管理详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:banner:query")
    @GetMapping("/{id}")
    public R<ZsBannerVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iZsBannerService.queryById(id));
    }

    /**
     * 新增站点banner管理
     */
    @SaCheckPermission("system:banner:add")
    @Log(title = "站点banner管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody ZsBannerBo bo) {
        return toAjax(iZsBannerService.insertByBo(bo));
    }

    /**
     * 修改站点banner管理
     */
    @SaCheckPermission("system:banner:edit")
    @Log(title = "站点banner管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody ZsBannerBo bo) {
        return toAjax(iZsBannerService.updateByBo(bo));
    }

    /**
     * 删除站点banner管理
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:banner:remove")
    @Log(title = "站点banner管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iZsBannerService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
