package com.ruoyi.system.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.ZsMessagesVo;
import com.ruoyi.system.domain.bo.ZsMessagesBo;
import com.ruoyi.system.service.IZsMessagesService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 站点留言
 *
 * @author 王强
 * @date 2025-05-06
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/messages")
public class ZsMessagesController extends BaseController {

    private final IZsMessagesService iZsMessagesService;

    /**
     * 查询站点留言列表
     */
    @SaCheckPermission("system:messages:list")
    @GetMapping("/list")
    public TableDataInfo<ZsMessagesVo> list(ZsMessagesBo bo, PageQuery pageQuery) {
        return iZsMessagesService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出站点留言列表
     */
    @SaCheckPermission("system:messages:export")
    @Log(title = "站点留言", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(ZsMessagesBo bo, HttpServletResponse response) {
        List<ZsMessagesVo> list = iZsMessagesService.queryList(bo);
        ExcelUtil.exportExcel(list, "站点留言", ZsMessagesVo.class, response);
    }

    /**
     * 获取站点留言详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:messages:query")
    @GetMapping("/{id}")
    public R<ZsMessagesVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iZsMessagesService.queryById(id));
    }

    /**
     * 新增站点留言
     */
    @SaCheckPermission("system:messages:add")
    @Log(title = "站点留言", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody ZsMessagesBo bo) {
        return toAjax(iZsMessagesService.insertByBo(bo));
    }

    /**
     * 修改站点留言
     */
    @SaCheckPermission("system:messages:edit")
    @Log(title = "站点留言", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody ZsMessagesBo bo) {
        return toAjax(iZsMessagesService.updateByBo(bo));
    }

    /**
     * 删除站点留言
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:messages:remove")
    @Log(title = "站点留言", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iZsMessagesService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
