package com.ruoyi.system.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.ZsOperationSmtpVo;
import com.ruoyi.system.domain.bo.ZsOperationSmtpBo;
import com.ruoyi.system.service.IZsOperationSmtpService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 邮件配置
 *
 * @author 王强
 * @date 2025-05-09
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/operationSmtp")
public class ZsOperationSmtpController extends BaseController {

    private final IZsOperationSmtpService iZsOperationSmtpService;

    /**
     * 查询邮件配置列表
     */
    @SaCheckPermission("system:operationSmtp:list")
    @GetMapping("/list")
    public TableDataInfo<ZsOperationSmtpVo> list(ZsOperationSmtpBo bo, PageQuery pageQuery) {
        return iZsOperationSmtpService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出邮件配置列表
     */
    @SaCheckPermission("system:operationSmtp:export")
    @Log(title = "邮件配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(ZsOperationSmtpBo bo, HttpServletResponse response) {
        List<ZsOperationSmtpVo> list = iZsOperationSmtpService.queryList(bo);
        ExcelUtil.exportExcel(list, "邮件配置", ZsOperationSmtpVo.class, response);
    }

    /**
     * 获取邮件配置详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:operationSmtp:query")
    @GetMapping("/{id}")
    public R<ZsOperationSmtpVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iZsOperationSmtpService.queryById(id));
    }

    /**
     * 新增邮件配置
     */
    @SaCheckPermission("system:operationSmtp:add")
    @Log(title = "邮件配置", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody ZsOperationSmtpBo bo) {
        return toAjax(iZsOperationSmtpService.insertByBo(bo));
    }

    /**
     * 修改邮件配置
     */
    @SaCheckPermission("system:operationSmtp:edit")
    @Log(title = "邮件配置", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody ZsOperationSmtpBo bo) {
        return toAjax(iZsOperationSmtpService.updateByBo(bo));
    }

    /**
     * 删除邮件配置
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:operationSmtp:remove")
    @Log(title = "邮件配置", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iZsOperationSmtpService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
