package com.ruoyi.system.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.ZsProductCategoryVo;
import com.ruoyi.system.domain.bo.ZsProductCategoryBo;
import com.ruoyi.system.service.IZsProductCategoryService;

/**
 * 站点产品分类管理
 *
 * @author 王强
 * @date 2025-05-06
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/productCategory")
public class ZsProductCategoryController extends BaseController {

    private final IZsProductCategoryService iZsProductCategoryService;

    /**
     * 查询站点产品分类管理列表
     */
    @SaCheckPermission("system:productCategory:list")
    @GetMapping("/list")
    public R<List<ZsProductCategoryVo>> list(ZsProductCategoryBo bo) {
        List<ZsProductCategoryVo> list = iZsProductCategoryService.queryList(bo);
        return R.ok(list);
    }

    /**
     * 导出站点产品分类管理列表
     */
    @SaCheckPermission("system:productCategory:export")
    @Log(title = "站点产品分类管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(ZsProductCategoryBo bo, HttpServletResponse response) {
        List<ZsProductCategoryVo> list = iZsProductCategoryService.queryList(bo);
        ExcelUtil.exportExcel(list, "站点产品分类管理", ZsProductCategoryVo.class, response);
    }

    /**
     * 获取站点产品分类管理详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:productCategory:query")
    @GetMapping("/{id}")
    public R<ZsProductCategoryVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iZsProductCategoryService.queryById(id));
    }

    /**
     * 新增站点产品分类管理
     */
    @SaCheckPermission("system:productCategory:add")
    @Log(title = "站点产品分类管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody ZsProductCategoryBo bo) {
        return toAjax(iZsProductCategoryService.insertByBo(bo));
    }

    /**
     * 修改站点产品分类管理
     */
    @SaCheckPermission("system:productCategory:edit")
    @Log(title = "站点产品分类管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody ZsProductCategoryBo bo) {
        return toAjax(iZsProductCategoryService.updateByBo(bo));
    }

    /**
     * 删除站点产品分类管理
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:productCategory:remove")
    @Log(title = "站点产品分类管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iZsProductCategoryService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
