package com.ruoyi.system.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.ZsPurposeVo;
import com.ruoyi.system.domain.bo.ZsPurposeBo;
import com.ruoyi.system.service.IZsPurposeService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 站点产品用途管理
 *
 * @author 王强
 * @date 2025-04-29
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/purpose")
public class ZsPurposeController extends BaseController {

    private final IZsPurposeService iZsPurposeService;

    /**
     * 查询站点产品用途管理列表
     */
    @SaCheckPermission("system:purpose:list")
    @GetMapping("/list")
    public TableDataInfo<ZsPurposeVo> list(ZsPurposeBo bo, PageQuery pageQuery) {
        return iZsPurposeService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出站点产品用途管理列表
     */
    @SaCheckPermission("system:purpose:export")
    @Log(title = "站点产品用途管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(ZsPurposeBo bo, HttpServletResponse response) {
        List<ZsPurposeVo> list = iZsPurposeService.queryList(bo);
        ExcelUtil.exportExcel(list, "站点产品用途管理", ZsPurposeVo.class, response);
    }

    /**
     * 获取站点产品用途管理详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:purpose:query")
    @GetMapping("/{id}")
    public R<ZsPurposeVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iZsPurposeService.queryById(id));
    }

    /**
     * 新增站点产品用途管理
     */
    @SaCheckPermission("system:purpose:add")
    @Log(title = "站点产品用途管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody ZsPurposeBo bo) {
        return toAjax(iZsPurposeService.insertByBo(bo));
    }

    /**
     * 修改站点产品用途管理
     */
    @SaCheckPermission("system:purpose:edit")
    @Log(title = "站点产品用途管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody ZsPurposeBo bo) {
        return toAjax(iZsPurposeService.updateByBo(bo));
    }

    /**
     * 删除站点产品用途管理
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:purpose:remove")
    @Log(title = "站点产品用途管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iZsPurposeService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
