package com.ruoyi.system.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.ZsWebsiteVo;
import com.ruoyi.system.domain.bo.ZsWebsiteBo;
import com.ruoyi.system.service.IZsWebsiteService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 站点信息
 *
 * @author 王强
 * @date 2025-04-30
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/website")
public class ZsWebsiteController extends BaseController {

    private final IZsWebsiteService iZsWebsiteService;

    /**
     * 查询站点信息列表
     */
    @SaCheckPermission("system:website:list")
    @GetMapping("/list")
    public TableDataInfo<ZsWebsiteVo> list(ZsWebsiteBo bo, PageQuery pageQuery) {
        return iZsWebsiteService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出站点信息列表
     */
    @SaCheckPermission("system:website:export")
    @Log(title = "站点信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(ZsWebsiteBo bo, HttpServletResponse response) {
        List<ZsWebsiteVo> list = iZsWebsiteService.queryList(bo);
        ExcelUtil.exportExcel(list, "站点信息", ZsWebsiteVo.class, response);
    }

    /**
     * 获取站点信息详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:website:query")
    @GetMapping("/{id}")
    public R<ZsWebsiteVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iZsWebsiteService.queryById(id));
    }

    /**
     * 新增站点信息
     */
    @SaCheckPermission("system:website:add")
    @Log(title = "站点信息", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody ZsWebsiteBo bo) {
        return toAjax(iZsWebsiteService.insertByBo(bo));
    }

    /**
     * 修改站点信息
     */
    @SaCheckPermission("system:website:edit")
    @Log(title = "站点信息", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody ZsWebsiteBo bo) {
        return toAjax(iZsWebsiteService.updateByBo(bo));
    }

    /**
     * 删除站点信息
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:website:remove")
    @Log(title = "站点信息", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iZsWebsiteService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
