package com.ruoyi.system.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.system.domain.bo.ZsArticleBo;
import com.ruoyi.system.domain.vo.ZsArticleVo;
import com.ruoyi.system.domain.ZsArticle;
import com.ruoyi.system.mapper.ZsArticleMapper;
import com.ruoyi.system.service.IZsArticleService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 站点文章管理Service业务层处理
 *
 * @author 王强
 * @date 2025-04-29
 */
@RequiredArgsConstructor
@Service
public class ZsArticleServiceImpl implements IZsArticleService {

    private final ZsArticleMapper baseMapper;

    /**
     * 查询站点文章管理
     */
    @Override
    public ZsArticleVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询站点文章管理列表
     */
    @Override
    public TableDataInfo<ZsArticleVo> queryPageList(ZsArticleBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZsArticle> lqw = buildQueryWrapper(bo);
        Page<ZsArticleVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询站点文章管理列表
     */
    @Override
    public List<ZsArticleVo> queryList(ZsArticleBo bo) {
        LambdaQueryWrapper<ZsArticle> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZsArticle> buildQueryWrapper(ZsArticleBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZsArticle> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getWebsiteName()), ZsArticle::getWebsiteName, bo.getWebsiteName());
        lqw.eq(StringUtils.isNotBlank(bo.getLanguageCode()), ZsArticle::getLanguageCode, bo.getLanguageCode());
        lqw.like(StringUtils.isNotBlank(bo.getTitle()), ZsArticle::getTitle, bo.getTitle());
        lqw.eq(StringUtils.isNotBlank(bo.getStatus()), ZsArticle::getStatus, bo.getStatus());
        lqw.eq(StringUtils.isNotBlank(bo.getIsDisabled()), ZsArticle::getIsDisabled, bo.getIsDisabled());
        return lqw;
    }

    /**
     * 新增站点文章管理
     */
    @Override
    public Boolean insertByBo(ZsArticleBo bo) {
        ZsArticle add = BeanUtil.toBean(bo, ZsArticle.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改站点文章管理
     */
    @Override
    public Boolean updateByBo(ZsArticleBo bo) {
        ZsArticle update = BeanUtil.toBean(bo, ZsArticle.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZsArticle entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除站点文章管理
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
