package com.ruoyi.system.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.system.domain.bo.ZsBannerBo;
import com.ruoyi.system.domain.vo.ZsBannerVo;
import com.ruoyi.system.domain.ZsBanner;
import com.ruoyi.system.mapper.ZsBannerMapper;
import com.ruoyi.system.service.IZsBannerService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 站点banner管理Service业务层处理
 *
 * @author ruoyi
 * @date 2025-04-28
 */
@RequiredArgsConstructor
@Service
public class ZsBannerServiceImpl implements IZsBannerService {

    private final ZsBannerMapper baseMapper;

    /**
     * 查询站点banner管理
     */
    @Override
    public ZsBannerVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询站点banner管理列表
     */
    @Override
    public TableDataInfo<ZsBannerVo> queryPageList(ZsBannerBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZsBanner> lqw = buildQueryWrapper(bo);
        Page<ZsBannerVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询站点banner管理列表
     */
    @Override
    public List<ZsBannerVo> queryList(ZsBannerBo bo) {
        LambdaQueryWrapper<ZsBanner> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZsBanner> buildQueryWrapper(ZsBannerBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZsBanner> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getWebsiteName()), ZsBanner::getWebsiteName, bo.getWebsiteName());
        lqw.eq(StringUtils.isNotBlank(bo.getLanguageCode()), ZsBanner::getLanguageCode, bo.getLanguageCode());
        lqw.like(StringUtils.isNotBlank(bo.getTitle()), ZsBanner::getTitle, bo.getTitle());
        lqw.eq(bo.getIsDisabled() != null, ZsBanner::getIsDisabled, bo.getIsDisabled());
        return lqw;
    }

    /**
     * 新增站点banner管理
     */
    @Override
    public Boolean insertByBo(ZsBannerBo bo) {
        ZsBanner add = BeanUtil.toBean(bo, ZsBanner.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改站点banner管理
     */
    @Override
    public Boolean updateByBo(ZsBannerBo bo) {
        ZsBanner update = BeanUtil.toBean(bo, ZsBanner.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZsBanner entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除站点banner管理
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
