package com.ruoyi.system.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.system.domain.bo.ZsOperationWarnconfigBo;
import com.ruoyi.system.domain.vo.ZsOperationWarnconfigVo;
import com.ruoyi.system.domain.ZsOperationWarnconfig;
import com.ruoyi.system.mapper.ZsOperationWarnconfigMapper;
import com.ruoyi.system.service.IZsOperationWarnconfigService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 预警配置Service业务层处理
 *
 * @author 王强
 * @date 2025-05-08
 */
@RequiredArgsConstructor
@Service
public class ZsOperationWarnconfigServiceImpl implements IZsOperationWarnconfigService {

    private final ZsOperationWarnconfigMapper baseMapper;

    /**
     * 查询预警配置
     */
    @Override
    public ZsOperationWarnconfigVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询预警配置列表
     */
    @Override
    public TableDataInfo<ZsOperationWarnconfigVo> queryPageList(ZsOperationWarnconfigBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZsOperationWarnconfig> lqw = buildQueryWrapper(bo);
        Page<ZsOperationWarnconfigVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询预警配置列表
     */
    @Override
    public List<ZsOperationWarnconfigVo> queryList(ZsOperationWarnconfigBo bo) {
        LambdaQueryWrapper<ZsOperationWarnconfig> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZsOperationWarnconfig> buildQueryWrapper(ZsOperationWarnconfigBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZsOperationWarnconfig> lqw = Wrappers.lambdaQuery();
        return lqw;
    }

    /**
     * 新增预警配置
     */
    @Override
    public Boolean insertByBo(ZsOperationWarnconfigBo bo) {
        ZsOperationWarnconfig add = BeanUtil.toBean(bo, ZsOperationWarnconfig.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改预警配置
     */
    @Override
    public Boolean updateByBo(ZsOperationWarnconfigBo bo) {
        ZsOperationWarnconfig update = BeanUtil.toBean(bo, ZsOperationWarnconfig.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZsOperationWarnconfig entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除预警配置
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
