package com.ruoyi.system.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.system.domain.bo.ZsProductCategoryBo;
import com.ruoyi.system.domain.vo.ZsProductCategoryVo;
import com.ruoyi.system.domain.ZsProductCategory;
import com.ruoyi.system.mapper.ZsProductCategoryMapper;
import com.ruoyi.system.service.IZsProductCategoryService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 站点产品分类管理Service业务层处理
 *
 * @author 王强
 * @date 2025-05-06
 */
@RequiredArgsConstructor
@Service
public class ZsProductCategoryServiceImpl implements IZsProductCategoryService {

    private final ZsProductCategoryMapper baseMapper;

    /**
     * 查询站点产品分类管理
     */
    @Override
    public ZsProductCategoryVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }


    /**
     * 查询站点产品分类管理列表
     */
    @Override
    public List<ZsProductCategoryVo> queryList(ZsProductCategoryBo bo) {
        LambdaQueryWrapper<ZsProductCategory> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZsProductCategory> buildQueryWrapper(ZsProductCategoryBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZsProductCategory> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getWebsiteName()), ZsProductCategory::getWebsiteName, bo.getWebsiteName());
        lqw.eq(StringUtils.isNotBlank(bo.getLanguageCode()), ZsProductCategory::getLanguageCode, bo.getLanguageCode());
        lqw.like(StringUtils.isNotBlank(bo.getName()), ZsProductCategory::getName, bo.getName());
        return lqw;
    }

    /**
     * 新增站点产品分类管理
     */
    @Override
    public Boolean insertByBo(ZsProductCategoryBo bo) {
        ZsProductCategory add = BeanUtil.toBean(bo, ZsProductCategory.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改站点产品分类管理
     */
    @Override
    public Boolean updateByBo(ZsProductCategoryBo bo) {
        ZsProductCategory update = BeanUtil.toBean(bo, ZsProductCategory.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZsProductCategory entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除站点产品分类管理
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
