package com.ruoyi.system.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.system.domain.bo.ZsProductBo;
import com.ruoyi.system.domain.vo.ZsProductVo;
import com.ruoyi.system.domain.ZsProduct;
import com.ruoyi.system.mapper.ZsProductMapper;
import com.ruoyi.system.service.IZsProductService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 站点产品管理Service业务层处理
 *
 * @author 王强
 * @date 2025-04-30
 */
@RequiredArgsConstructor
@Service
public class ZsProductServiceImpl implements IZsProductService {

    private final ZsProductMapper baseMapper;

    /**
     * 查询站点产品管理
     */
    @Override
    public ZsProductVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询站点产品管理列表
     */
    @Override
    public TableDataInfo<ZsProductVo> queryPageList(ZsProductBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZsProduct> lqw = buildQueryWrapper(bo);
        Page<ZsProductVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询站点产品管理列表
     */
    @Override
    public List<ZsProductVo> queryList(ZsProductBo bo) {
        LambdaQueryWrapper<ZsProduct> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZsProduct> buildQueryWrapper(ZsProductBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZsProduct> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getWebsiteName()), ZsProduct::getWebsiteName, bo.getWebsiteName());
        lqw.eq(StringUtils.isNotBlank(bo.getLanguageCode()), ZsProduct::getLanguageCode, bo.getLanguageCode());
        lqw.eq(bo.getCategoryId() != null, ZsProduct::getCategoryId, bo.getCategoryId());
        lqw.like(StringUtils.isNotBlank(bo.getName()), ZsProduct::getName, bo.getName());
        lqw.eq(StringUtils.isNotBlank(bo.getIsDisabled()), ZsProduct::getIsDisabled, bo.getIsDisabled());
        return lqw;
    }

    /**
     * 新增站点产品管理
     */
    @Override
    public Boolean insertByBo(ZsProductBo bo) {
        ZsProduct add = BeanUtil.toBean(bo, ZsProduct.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改站点产品管理
     */
    @Override
    public Boolean updateByBo(ZsProductBo bo) {
        ZsProduct update = BeanUtil.toBean(bo, ZsProduct.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZsProduct entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除站点产品管理
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
