package com.ruoyi.system.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.system.domain.bo.ZsPurposeBo;
import com.ruoyi.system.domain.vo.ZsPurposeVo;
import com.ruoyi.system.domain.ZsPurpose;
import com.ruoyi.system.mapper.ZsPurposeMapper;
import com.ruoyi.system.service.IZsPurposeService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 站点产品用途管理Service业务层处理
 *
 * @author 王强
 * @date 2025-04-29
 */
@RequiredArgsConstructor
@Service
public class ZsPurposeServiceImpl implements IZsPurposeService {

    private final ZsPurposeMapper baseMapper;

    /**
     * 查询站点产品用途管理
     */
    @Override
    public ZsPurposeVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询站点产品用途管理列表
     */
    @Override
    public TableDataInfo<ZsPurposeVo> queryPageList(ZsPurposeBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZsPurpose> lqw = buildQueryWrapper(bo);
        Page<ZsPurposeVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询站点产品用途管理列表
     */
    @Override
    public List<ZsPurposeVo> queryList(ZsPurposeBo bo) {
        LambdaQueryWrapper<ZsPurpose> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZsPurpose> buildQueryWrapper(ZsPurposeBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZsPurpose> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getWebsiteName()), ZsPurpose::getWebsiteName, bo.getWebsiteName());
        lqw.eq(StringUtils.isNotBlank(bo.getLanguageCode()), ZsPurpose::getLanguageCode, bo.getLanguageCode());
        lqw.like(StringUtils.isNotBlank(bo.getName()), ZsPurpose::getName, bo.getName());
        return lqw;
    }

    /**
     * 新增站点产品用途管理
     */
    @Override
    public Boolean insertByBo(ZsPurposeBo bo) {
        ZsPurpose add = BeanUtil.toBean(bo, ZsPurpose.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改站点产品用途管理
     */
    @Override
    public Boolean updateByBo(ZsPurposeBo bo) {
        ZsPurpose update = BeanUtil.toBean(bo, ZsPurpose.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZsPurpose entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除站点产品用途管理
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
