package com.ruoyi.system.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.system.domain.bo.ZsWebsiteBo;
import com.ruoyi.system.domain.vo.ZsWebsiteVo;
import com.ruoyi.system.domain.ZsWebsite;
import com.ruoyi.system.mapper.ZsWebsiteMapper;
import com.ruoyi.system.service.IZsWebsiteService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 站点信息Service业务层处理
 *
 * @author 王强
 * @date 2025-04-30
 */
@RequiredArgsConstructor
@Service
public class ZsWebsiteServiceImpl implements IZsWebsiteService {

    private final ZsWebsiteMapper baseMapper;

    /**
     * 查询站点信息
     */
    @Override
    public ZsWebsiteVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询站点信息列表
     */
    @Override
    public TableDataInfo<ZsWebsiteVo> queryPageList(ZsWebsiteBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZsWebsite> lqw = buildQueryWrapper(bo);
        Page<ZsWebsiteVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询站点信息列表
     */
    @Override
    public List<ZsWebsiteVo> queryList(ZsWebsiteBo bo) {
        LambdaQueryWrapper<ZsWebsite> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZsWebsite> buildQueryWrapper(ZsWebsiteBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZsWebsite> lqw = Wrappers.lambdaQuery();
        lqw.eq(StringUtils.isNotBlank(bo.getLanguageCode()), ZsWebsite::getLanguageCode, bo.getLanguageCode());
        lqw.like(StringUtils.isNotBlank(bo.getSiteName()), ZsWebsite::getSiteName, bo.getSiteName());
        lqw.like(StringUtils.isNotBlank(bo.getCode()), ZsWebsite::getCode, bo.getCode());
        return lqw;
    }

    /**
     * 新增站点信息
     */
    @Override
    public Boolean insertByBo(ZsWebsiteBo bo) {
        ZsWebsite add = BeanUtil.toBean(bo, ZsWebsite.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改站点信息
     */
    @Override
    public Boolean updateByBo(ZsWebsiteBo bo) {
        ZsWebsite update = BeanUtil.toBean(bo, ZsWebsite.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZsWebsite entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除站点信息
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
